import imghdr
import os.path
import warnings
import numpy as np
from pathlib import Path


def train_test_split(dataset_root: Path, val_size=0.2, test_size=0, separator=" "):
    """
    数据集划分
    :param dataset_root:
    :param val_size:
    :param test_size:
    :param separator: 分割符
    :return:
    """
    split = [1 - val_size - test_size, val_size, test_size]
    image_dir = dataset_root.joinpath("images")
    label_dir = dataset_root.joinpath("labels")
    image_files = []
    for f in image_dir.glob("*.*"):
        if not imghdr.what(f):
            continue
        image_files.append(str(f))
    label_files = []
    for f in label_dir.glob("*.png"):
        label_files.append(str(f))

    if not image_files:
        warnings.warn("No files in {}".format(image_dir))
    if not label_files:
        warnings.warn("No files in {}".format(label_dir))

    num_images = len(image_files)
    num_label = len(label_files)
    if num_images != num_label:
        raise Exception(
            "Number of images = {}, number of labels = {}."
            "The number of images is not equal to number of labels, "
            "Please check your dataset!".format(num_images, num_label))

    image_files = np.array(image_files)
    label_files = np.array(label_files)
    state = np.random.get_state()
    np.random.shuffle(image_files)
    np.random.set_state(state)
    np.random.shuffle(label_files)
    start = 0
    num_split = len(split)
    dataset_name = ['train', 'val', 'test']
    for i in range(num_split):
        if split[i] == 0:
            continue
        dataset_split = dataset_name[i]
        print("Creating {}.txt...".format(dataset_split))
        if split[i] > 1.0 or split[i] < 0:
            raise ValueError("{} dataset percentage should be 0~1.".format(
                dataset_split))

        file_list = os.path.join(str(dataset_root), dataset_split + '_list.txt')
        with open(file_list, "w") as f:
            num = round(split[i] * num_images)
            end = start + num
            if i == num_split - 1:
                end = num_images
            for item in range(start, end):
                left = image_files[item].replace(str(dataset_root), '')
                left = left.replace("\\", '/')[1:]
                if left[0] == os.path.sep:
                    left = left.lstrip(os.path.sep)

                try:
                    right = label_files[item].replace(str(dataset_root), '')
                    right = right.replace("\\", '/')[1:]
                    if right[0] == os.path.sep:
                        right = right.lstrip(os.path.sep)
                    line = left + separator + right + '\n'
                except:
                    line = left + '\n'

                f.write(line)
                print(line)
            start = end


if __name__ == '__main__':
    pass



# -*- coding:utf-8 -*-
# Author:  zhousf
# Description: 目标检测bbox计算工具
# pip install matplotlib
import matplotlib.pyplot as plt
import matplotlib.colors as mcolors


def show_rect(boxes: list):
    """
    显示box
    :param boxes: [(x_min, y_min, x_max, y_max)]
    :return:
    [(317,280,553,395), (374,295,485,322)]
    """
    colors = list(mcolors.TABLEAU_COLORS.keys())
    plt.xlabel("x", fontweight='bold', size=14)
    plt.ylabel("y", fontweight='bold', size=14)
    ax = plt.gca()  # 坐标系
    x_max = 0
    y_max = 0
    for index, box in enumerate(boxes):
        x_max = box[2] if box[2] > x_max else x_max
        y_max = box[3] if box[3] > y_max else y_max
        ax.add_patch(
            plt.Rectangle(xy=(box[0], box[1]), width=(box[2] - box[0]), height=(box[3] - box[1]),
                          alpha=1,
                          fill=False,
                          color=colors[index],
                          facecolor=colors[index],
                          linewidth=1))
    plt.xlim(0, int(2 * x_max))
    plt.ylim(0, int(2 * y_max))
    # 转成屏幕坐标系（左上角为原点）
    ax.xaxis.set_ticks_position('top')  # 将X坐标轴移到上面
    ax.invert_yaxis()  # 反转Y坐标轴
    plt.show()


def compute_iou(predicted_box, ground_truth_box):
    """
    计算交并比
    :param predicted_box: 预测box=(x_min, y_min, x_max, y_max)
    :param ground_truth_box: 真实box=(x_min, y_min, x_max, y_max)
    :return:
    """
    px_min, py_min, px_max, py_max = predicted_box
    gx_min, gy_min, gx_max, gy_max = ground_truth_box
    p_area = (px_max - px_min) * (py_max - py_min)  # 计算P的面积
    g_area = (gx_max - gx_min) * (gy_max - gy_min)  # 计算G的面积
    # 求相交矩形的左下和右上顶点坐标(x_min, y_min, x_max, y_max)
    _x_min = max(px_min, gx_min)  # 得到左下顶点的横坐标
    _y_min = max(py_min, gy_min)  # 得到左下顶点的纵坐标
    _x_max = min(px_max, gx_max)  # 得到右上顶点的横坐标
    _y_max = min(py_max, gy_max)  # 得到右上顶点的纵坐标
    # 计算相交矩形的面积
    w = _x_max - _x_min
    h = _y_max - _y_min
    if w <= 0 or h <= 0:
        return 0
    area = w * h  # G∩P的面积
    return area / (p_area + g_area - area)


def compute_contain(box1, box2):
    """
    计算两个box是否为包含关系
    :param box1: (x_min, y_min, x_max, y_max)
    :param box2: (x_min, y_min, x_max, y_max)
    :return: 返回两个box重叠面积占较小box的面积比，一般大于0.8则为包含关系
    box1=(317,280,553,395)
    box2=(374,295,485,322)
    """
    px_min = min(box1[0], box1[2])
    py_min = min(box1[1], box1[3])
    px_max = max(box1[0], box1[2])
    py_max = max(box1[1], box1[3])
    # px_min, py_min, px_max, py_max = box1
    # gx_min, gy_min, gx_max, gy_max = box2
    gx_min = min(box2[0], box2[2])
    gy_min = min(box2[1], box2[3])
    gx_max = max(box2[0], box2[2])
    gy_max = max(box2[1], box2[3])
    p_area = (px_max - px_min) * (py_max - py_min)  # 计算P的面积
    g_area = (gx_max - gx_min) * (gy_max - gy_min)  # 计算G的面积
    # 求相交矩形的左下和右上顶点坐标(x_min, y_min, x_max, y_max)
    _x_min = max(px_min, gx_min)  # 得到左下顶点的横坐标
    _y_min = max(py_min, gy_min)  # 得到左下顶点的纵坐标
    _x_max = min(px_max, gx_max)  # 得到右上顶点的横坐标
    _y_max = min(py_max, gy_max)  # 得到右上顶点的纵坐标
    # 计算相交矩形的面积
    w = _x_max - _x_min
    h = _y_max - _y_min
    if w <= 0 or h <= 0:
        return 0
    area = w * h  # G∩P的面积
    if p_area >= g_area:
        return area / g_area
    else:
        return area / p_area


def group_by_box_overlap(od_result: list, return_area=False, area_rate=0.8):
    """
    根据box的重叠面积进行分组-通用算法，适用目标检测、画图等
    :param od_result: [(?, ?, x_min, y_min, x_max, y_max)], box置于tuple最后
    :param return_area: 是否返回面积
    :param area_rate: 两个box重叠面积比例，若大于该值则为包含关系
    :return:
    [[(index, [area, (?, ?, x_min, y_min, x_max, y_max)])]]
    or
    [[(?, ?, x_min, y_min, x_max, y_max)]]
    """
    boxes = {}
    # 按照面积从大到小排序, box置于tuple最后
    for index, item in enumerate(od_result):
        (x_min, y_min, x_max, y_max) = item[-4:]
        area = (x_max - x_min) * (y_max - y_min)
        boxes[index] = [area, item]
    boxes = sorted(boxes.items(), key=lambda d: d[1], reverse=True)
    box_group = []
    has_add_index = []
    for item1 in boxes:
        (index1, [area1, box1]) = item1
        (x_min1, y_min1, x_max1, y_max1) = box1[-4:]
        items = [item1] if return_area else [box1]
        if index1 in has_add_index:
            continue
        has_add_index.append(index1)
        for i, item2 in enumerate(boxes):
            (index2, [area2, box2]) = item2
            (x_min2, y_min2, x_max2, y_max2) = box2[-4:]
            if compute_contain((x_min1, y_min1, x_max1, y_max1),
                               (x_min2, y_min2, x_max2, y_max2)) > area_rate:
                if item1 == item2:
                    continue
                if index2 in has_add_index:
                    continue
                has_add_index.append(index2)
                if return_area:
                    items.append(item2)
                else:
                    items.append(box2)
        box_group.append(items)
    return box_group


def search_right_box(boxes: list):
    """
    搜索最右侧box
    :param boxes: [(?, ?, x_min, y_min, x_max, y_max)]
    :return: box
    """
    if len(boxes) == 0:
        return None
    boxes.sort(key=lambda x: x[-2], reverse=True)
    return boxes[0]


def search_top_box(boxes: list):
    """
    搜索最顶端box
    :param boxes: [(?, ?, x_min, y_min, x_max, y_max)]
    :return: box
    """
    if len(boxes) == 0:
        return None
    boxes.sort(key=lambda x: x[-3], reverse=False)
    return boxes[0]


def search_bottom_box(boxes: list):
    """
    搜索最底端box
    :param boxes: [(?, ?, x_min, y_min, x_max, y_max)]
    :return: box
    """
    if len(boxes) == 0:
        return None
    boxes.sort(key=lambda x: x[-1], reverse=True)
    return boxes[0]


def search_nearby_bottom_box(target_box, boxes: list):
    """
    搜索紧邻target_box底部的box
    :param target_box: (?, ?, x_min, y_min, x_max, y_max)
    :param boxes: [(?, ?, x_min, y_min, x_max, y_max)]
    :return: box
    """
    if len(boxes) == 0:
        return None
    t_x = (target_box[-2] + target_box[-4]) / 2
    t_y = (target_box[-3] + target_box[-1]) / 2
    t_width = abs(target_box[-2] - target_box[-4])
    t_height = abs(target_box[-3] - target_box[-1])
    for box in boxes:
        c_x = (box[-2] + box[-4]) / 2
        c_y = (box[-3] + box[-1]) / 2
        c_width = abs(box[-2] - box[-4])
        c_height = abs(box[-3] - box[-1])
        # 两个中心点的X轴坐标差不超过两个box的高度和的一半，表示两个box在同一垂直线上
        if abs(c_x-t_x) < (t_width + c_width) / 2:
            if t_y < c_y:
                return box
    return None


def search_nearby_right_box(target_box, boxes: list):
    """
    搜索紧邻target_box右侧的box
    :param target_box: (?, ?, x_min, y_min, x_max, y_max)
    :param boxes: [(?, ?, x_min, y_min, x_max, y_max)]
    :return: box
    """
    if len(boxes) == 0:
        return None
    t_x = (target_box[-2] + target_box[-4]) / 2
    t_y = (target_box[-3] + target_box[-1]) / 2
    t_width = abs(target_box[-2] - target_box[-4])
    t_height = abs(target_box[-3] - target_box[-1])
    for box in boxes:
        c_x = (box[-2] + box[-4]) / 2
        c_y = (box[-3] + box[-1]) / 2
        c_width = abs(box[-2] - box[-4])
        c_height = abs(box[-3] - box[-1])
        # 两个中心点的Y轴坐标差不超过两个box的高度和的一半，表示两个box在同一水平线上
        if abs(c_y-t_y) < (t_height + c_height) / 2:
            if t_x < c_x:
                return box
    return None


def search_nearby_left_box(target_box, boxes: list):
    """
    搜索紧邻target_box左侧的box
    :param target_box: (?, ?, x_min, y_min, x_max, y_max)
    :param boxes: [(?, ?, x_min, y_min, x_max, y_max)]
    :return: box
    """
    if len(boxes) == 0:
        return None
    t_x = (target_box[-2] + target_box[-4]) / 2
    t_y = (target_box[-3] + target_box[-1]) / 2
    t_width = abs(target_box[-2] - target_box[-4])
    t_height = abs(target_box[-3] - target_box[-1])
    for box in boxes:
        c_x = (box[-2] + box[-4]) / 2
        c_y = (box[-3] + box[-1]) / 2
        c_width = abs(box[-2] - box[-4])
        c_height = abs(box[-3] - box[-1])
        # 两个中心点的Y轴坐标差不超过两个box的高度和的一半，表示两个box在同一水平线上
        if abs(c_y-t_y) < (t_height + c_height) / 2:
            if t_x > c_x:
                return box
    return None


def box_scale_up(box, offset=50):
    """
    box增大
    :param box:
    :param offset:
    :return:
    """
    x_min, y_min, x_max, y_max = box
    _x_min = x_min - offset
    _x_min = 0 if _x_min < 0 else _x_min
    _x_max = x_max + offset
    _y_min = y_min - offset
    _y_min = 0 if _y_min < 0 else _y_min
    _y_max = y_max + offset
    return _x_min, _y_min, _x_max, _y_max


def box_scale_up_horizontal(box, offset=50):
    """
    box增大，仅水平方向
    :param box:
    :param offset:
    :return:
    """
    x_min, y_min, x_max, y_max = box
    _x_min = x_min - offset
    _x_min = 0 if _x_min < 0 else _x_min
    _x_max = x_max + offset
    return _x_min, y_min, _x_max, y_max


def box_scale_up_vertical(box, offset=50):
    """
    box增大，仅垂直方向
    :param box:
    :param offset:
    :return:
    """
    x_min, y_min, x_max, y_max = box
    _y_min = y_min - offset
    _y_min = 0 if _y_min < 0 else _y_min
    _y_max = y_max + offset
    return x_min, _y_min, x_max, _y_max


def box_scale_down(box, offset=50):
    """
    box缩小
    :param box:
    :param offset:
    :return:
    """
    x_min, y_min, x_max, y_max = box
    offset = min(x_min, y_min) if min(x_min, y_min) < offset else offset
    _x_min = x_min + offset
    _x_max = x_max - offset
    _y_min = y_min + offset
    _y_max = y_max - offset
    return _x_min, _y_min, _x_max, _y_max


def location_y_axis(target_box, box, height_threshold=0.25):
    """
    y轴位置关系判断，判断box在target_box的上面还是下面，通过中心点的y计算
    :param target_box:
    :param box:
    :param height_threshold: 阈值比，若大于target_box该比例的高度，则位置不同
    :return: 1: 上面   0: 下面   -1: 位置相同
    """
    c_y_t = (target_box[-3] + target_box[-1]) / 2
    height_t = abs(target_box[-3] - target_box[-1])
    c_y = (box[-3] + box[-1]) / 2
    # 两个中心点的Y轴坐标差超过target_box高度*height_threshold，表示两个box在target_box上面
    if abs(c_y_t - c_y) >= (height_t * height_threshold):
        if c_y_t > c_y:
            return 1
        else:
            return 0
    else:
        return -1


if __name__ == "__main__":
    # print(box_scale_down((10, 10, 20, 20), offset=2))
    # print(box_scale_up((-166.68197631835938, -0.008893102407455444, 1810.6822509765625, 143.40452575683594), offset=2))
    # a =(168.9995880126953, 40.77224349975586, 186.8643341064453, 62.222076416015625)
    # b =(151.0, 34.0, 234.0, 77.0)
    # print(compute_iou(a, b))
    # print(compute_contain(a, b))
    # show_rect([a, b])
    pass



# -*- coding:utf-8 -*-
# Author:      zhousf
# Date:        2019-05-30
# File:        math_util.py
# Description:
import math


def get_distance_from_point_to_point(point_first, point_second):
    """
    计算点到点的距离
    :param point_first:
    :param point_second:
    :return:
    """
    (p_x1, p_y1) = point_first
    (p_x2, p_y2) = point_second
    x = p_x1 - p_x2
    y = p_y2 - p_y1
    # 用math.sqrt求平方根 x**2->幂运算
    return math.sqrt((x**2) + (y**2))


def get_distance_from_point_to_line(point, line_first_point, line_second_point):
    """
    计算点到线的距离
    :param point:
    :param line_first_point:
    :param line_second_point:
    :return:
    """
    (point_x, point_y) = point
    (first_point_x, first_point_y) = line_first_point
    (second_point_x, second_point_y) = line_second_point
    a = second_point_y-first_point_y
    b = first_point_x-second_point_x
    c = second_point_x*first_point_y-first_point_x*second_point_y
    dis = (math.fabs(a*point_x+b*point_y+c)) / (math.pow(a*a+b*b, 0.5))
    return dis


if __name__ == '__main__':
    point = (0, 0)
    line_first_point = (-1, 1)
    line_second_point = (1, 1)
    # point = (269, 19)
    # line_first_point = (387, 24)
    # line_second_point = (228, 24)
    print(get_distance_from_point_to_line(point, line_first_point, line_second_point))
    # print(get_distance_from_point_to_point((47,196),(287,318)))


# -*- coding:utf-8 -*-
# Author:      zhousf
# Description:  批量异步下载工具类
"""
pip install grequests
"""
import os
import time
import grequests
from requests import Response


class DownloadBatch(object):

    def __init__(self, save_dir, concurrent=True):
        """
        批量下载文件工具类
        :param save_dir: 保存文件目录
        :param concurrent: True并行  False串行
        """
        self.save_dir = save_dir
        self.req_list = []
        if not os.path.exists(save_dir):
            os.makedirs(save_dir)
        self.consume_time = 0
        self.file_names = []
        self.task_num = 0
        self.concurrent = concurrent

    @staticmethod
    def exception_handler(request, exception):
        r = Response()
        r.status_code = 408
        r.reason = "download failed"
        return r

    def add(self, name, url, timeout=20.0):
        """

        :param name:
        :param url:
        :param timeout:
        :return:
        """
        if url is None or name is None:
            return False
        if name in self.file_names:
            return False
        self.file_names.append(name)
        self.req_list.append(grequests.get(url, timeout=timeout))
        return True

    def add_all(self, files):
        """

        :param files: [{"name": "1.jpg", "url": ""}]
        :return:
        """
        for file in files:
            name = file.get("name")
            url = file.get("url")
            if url is None or name is None:
                continue
            timeout = file.get("timeout", default=20.0)
            self.req_list.append(grequests.get(url, timeout=timeout))
            self.file_names.append(name)

    def run(self):
        """

        :return:
        """
        result = {}
        start = time.time()
        self.task_num = len(self.req_list)
        if self.task_num == 0:
            return result
        size = self.task_num if self.concurrent else 1
        responses = grequests.map(requests=self.req_list, size=size, exception_handler=self.exception_handler)
        for i in range(0, len(responses)):
            response = responses[i]
            try:
                save_file = "{0}/{1}".format(self.save_dir, self.file_names[i])
                target_type = self.file_names[i].split(".")[-1]
                if 'Content-Type' in response.headers:
                    content_type = response.headers['Content-Type']
                    if target_type in ["jpg", "JPG", "JPEG", "jpeg", "png", "PNG", "gif", "GIF"]:
                        target_type = "image"
                    if content_type.find(target_type) < 0:
                        result[self.file_names[i]] = (False,
                                                      "The file type is {0}, but {1} is expected. {2}".format(
                                                          content_type,
                                                          target_type,
                                                          response.text),
                                                      self.req_list[i].url)
                        continue
                if response.status_code != 200:
                    result[self.file_names[i]] = (
                        False, "{0}({1})".format(response.text, response.status_code), self.req_list[i].url)
                    continue
                # 当信息流小于100字节，则不是文件
                if len(response.text) <= 100:
                    result[self.file_names[i]] = (
                        False, "{0}({1})".format(response.text, response.status_code), self.req_list[i].url)
                    continue
                with open(save_file, "wb") as f:
                    f.write(response.content)
                    result[self.file_names[i]] = (True, save_file, self.req_list[i].url)
            except Exception as ex:
                result[self.file_names[i]] = (
                    False, "{0}({1})".format(response.text, response.status_code), self.req_list[i].url)
                continue
        end = time.time()
        self.consume_time = end - start
        return result


if __name__ == "__main__":
    url = ""
    downloader = DownloadBatch(save_dir="/home/ubuntu/Downloads")
    downloader.add(name="1.jpg", url=url, timeout=6)
    downloader.add(name="2.jpg", url=url, timeout=0.005)
    results = downloader.run()
    for result in results:
        success, save_file, url = results.get(result)
        print(result, success, save_file, url)
    print("cost time: {0}s | 共{1}项".format(downloader.consume_time, downloader.task_num))



# -*- coding: utf-8 -*-
# @Author  : zhousf
# @Function:
import numpy as np
import matplotlib.pyplot as plt
import matplotlib.patches as mpathes
from shapely import geometry


def in_poly(poly, point):
    """
    判断点是否在poly内，不包含poly边界
    :param poly: 多边形点坐标：[(0, 0), (1, 0), (1, 1), (0, 1)]
    :param point: 点坐标：(0.1, 0.5)
    :return:
    """
    line = geometry.LineString(poly)
    point = geometry.Point(point)
    polygon = geometry.Polygon(line)
    return polygon.contains(point)


def make_mesh(box_min, min_box_w, min_box_h, show=False):
    """
    按照w,h对box进行网格划分
    make_mesh([0, 0, 1280, 1280], 128, 128)
    :param box_min:
    :param min_box_w:
    :param min_box_h:
    :param show:
    :return:
    所有小box，按列排序 [(column_index, row_index, x_min, y_min, x_max, y_max)]
    column_index:列索引
    row_index:行索引
    """
    [x_min, y_min, x_max, y_max] = box_min
    list_x = np.arange(x_min, x_max, min_box_w)
    list_y = np.arange(y_min, y_max, min_box_h)
    # list_x中第i项和list_y中第j项所代表的网格为[list_x[i],list_y[j],list_x[i+1],list_y[j+1]]
    if show:
        fig, ax = plt.subplots()
        color = ['red', 'black', 'yellow', 'blue', 'green', 'purple']
    box_min = []  # x_min, y_min, x_max, y_max
    for i in range(len(list_x)):
        for j in range(len(list_y)):
            x_left = list_x[i]
            y_down = list_y[j]
            if i == len(list_x) - 1:
                x_right = x_max
            else:
                x_right = list_x[i + 1]
            if j == len(list_y) - 1:
                yup = y_max
            else:
                yup = list_y[j + 1]
            box_min.append((i, j, x_left, y_down, x_right, yup))
            if show:
                rect = mpathes.Rectangle((x_left, y_down), min_box_w, min_box_h, linewidth=1, edgecolor='r', facecolor='none')
                # rect = mpathes.Rectangle((x_left, y_down), min_box_w, min_box_h, color=color[(i + j % 5) % 5])
                ax.add_patch(rect)
    if show:
        ax.set_xbound(x_min, x_max)
        ax.set_ybound(y_min, y_max)
        plt.show()
    return box_min


def query_box_position(boxes, point):
    """
    # 查询point在哪个box中
    :param boxes:
    :param point:
    :return: (column_index, row_index, x_min, y_min, x_max, y_max)
    column_index:列索引
    row_index:行索引
    """
    for box in boxes:
        column_index, row_index, x_min, y_min, x_max, y_max = box
        # 将box外扩1个像素，使poly计算包含边界
        x_min -= 1
        y_min -= 1
        x_max += 1
        y_max += 1
        # 计算poly坐标
        poly = [(x_min, y_min), (x_max, y_min), (x_max, y_max), (x_min, y_max)]
        # 判断点是否在poly内，不包含poly边界
        if in_poly(poly=poly, point=point):
            return box
    return None


if __name__ == "__main__":
    # 判断点是否在poly内
    # poly = [(0, 0), (1, 0), (1, 1), (0, 1)]  # 多边形坐标
    # pt2 = (0.1, 0.5)  # 点坐标
    # print(in_poly(square, pt1))
    # 按照w,h对box进行网格划分
    boxes_ = make_mesh([0, 0, 4000, 5000], 1280, 1280, show=True)
    for box in boxes_:
        print(box)
        # [(column_index, row_index, x_min, y_min, x_max, y_max)]
    # 查询点在哪个box中
    # query_box_position(boxes=boxes_, point=(1, 1))
    pass



# -*- coding: utf-8 -*-
# @Author  : zhousf
# @Function:
import re


def get_digit_char(string: str):
    # 提取数字
    return re.sub(u"([^\u0030-\u0039])", "", string)


def only_contain_letter_char(self, string: str):
    """
    仅包含字母（大小写）
    """
    return len(self.get_letter_char(string)) == len(string)


def get_letter_char(string: str):
    # 提取大小写字母
    return re.sub(u"([^\u0041-\u005a\u0061-\u007a])", "", string)


def get_digit_letter_char(string: str):
    # 提取大小写字母、数字
    return re.sub(u"([^\u0041-\u005a\u0061-\u007a\u0030-\u0039])", "", string)


def only_chinese(string: str):
    """
    string都是中文
    :param string:
    :return: True都是中文 | 否
    """
    match_chinese = re.sub(u"([^\u4e00-\u9fa5])", "", string)
    return len(match_chinese) == len(string)


def normalize_cos_sign(string):
    """
    cos符号标准化：将cos15°转成cos(radians(15))
    cos15° -> cos(radians(15))
    cos5 -> cos(radians(5))
    cosa° -> cos(radians(a))
    cosθ -> cos(radians(θ))
    cos(a) -> cos(radians(a))
    cos(15°) -> cos(radians(15))
    cos(5) -> cos(radians(5))
    :param string:
    :return:
    """
    # ori = string
    items = re.split("[+-/*]", string)
    if len(items) > 0:
        for item in items:
            res = re.findall("[Cc][Oo0][Ss]", item)
            if len(res) > 0:
                value = item.replace(res[0], "").replace("°", "").replace("。", "").replace("(", "").replace(")", "")
                value = "(radians({0}))".format(value)
                string = string.replace(item, "cos" + value)
    # print(ori, "->", side_length)
    return string


def normalize_multiple_sign(string, normalize_char="*"):
    """
    乘号标准化：将x、X、×转成*
    161.9+x2-nXSxd+nxS+2x+2x(a+b) -> 161.9+x2-n*S*d+n*S+2x+2*(a+b)
    :param string:
    :param normalize_char: 标准化的字符
    :return:
    """
    items = re.split("[+-/*]", string)
    if len(items) > 0:
        for item in items:
            res = re.findall("^[^xX×].*[xX×]{1}.*[^xX×]$", item)
            if len(res) > 0:
                item = str(res[0]).replace("x", normalize_char).replace("X", normalize_char).replace("×", normalize_char)
                string = string.replace(res[0], item)
    return string




# -*- coding:utf-8 -*-
# Author:      zhousf
# Description:
import threading
import time


class MultiThread(threading.Thread):
    """
    多线程
    """
    def __init__(self, name, func, func_arg=None, thread_lock=None):
        """
        :param name: 线程名
        :param func: 执行函数
        :param thread_lock: threading.Lock() 默认异步
        """
        threading.Thread.__init__(self)
        self.name = name
        self.func = func
        self.thread_lock = thread_lock
        self.func_arg = func_arg
        self.result = None

    def run(self):
        if self.thread_lock:
            # 可选的timeout参数不填时将一直阻塞直到获得锁定
            # 成功获得锁定后返回True，超时返回False
            self.thread_lock.acquire()
        self.result = self.func(self.name, self.func_arg)
        if self.thread_lock:
            self.thread_lock.release()

    def get_result(self):
        try:
            return self.result
        except Exception as e:
            print(e)
            return None


def my_func(thread_name, func_arg):
    counter = 3
    while counter:
        time.sleep(1)
        print("{0}: {1}".format(thread_name, func_arg))
        counter -= 1
    return thread_name


def test_async():
    """
    异步
    """
    thread_1 = MultiThread(name="thread_1", func=my_func, func_arg="dddd1", thread_lock=None)
    thread_2 = MultiThread(name="thread_2", func=my_func, func_arg="32333", thread_lock=None)
    thread_1.start()
    thread_2.start()
    # 等待所有线程完成
    threads = [thread_1, thread_2]
    for t in threads:
        t.join()
        print(t.get_result())
    print("Exiting Main Thread")


def test_sync():
    """
    同步
    """
    thread_lock = threading.Lock()
    thread_1 = MultiThread(name="thread_1", func=my_func, thread_lock=thread_lock)
    thread_2 = MultiThread(name="thread_2", func=my_func, thread_lock=thread_lock)
    thread_1.start()
    thread_2.start()
    # 等待所有线程完成
    threads = [thread_1, thread_2]
    for t in threads:
        t.join()
    print("Exiting Main Thread")


if __name__ == "__main__":
    test_async()
    # test_sync()
    pass
